@@ -1,8 +1,8 @@
 #' @name Deriv
-#' @title Symbollic differentiation of an expression or function
-#' @description Symbollic differentiation of an expression or function
+#' @title Symbolic differentiation of an expression or function
+#' @description Symbolic differentiation of an expression or function
 #' @aliases Deriv drule
-#' @concept symbollic differentiation
+#' @concept symbolic differentiation
 #'
 #' @param f An expression or function to be differentiated.
 #'  f can be \itemize{
@@ -13,7 +13,7 @@
 #'   \item a language: \code{quote(x**n)}
 #'   \item a right hand side of a formula: \code{~ x**n} or \code{y ~ x**n}
 #'  }
-#' @param x An optional character vector with variable name(s) with resptect to which
+#' @param x An optional character vector with variable name(s) with respect to which
 #'  \code{f} must be differentiated. If not provided (i.e. x=NULL), x is
 #'  guessed either from\ code{names(formals(f))} (if \code{f} is a function)
 #'  or from all variables in f in other cases.
@@ -22,7 +22,7 @@
 #'  variables \code{x}
 #'  must be a named vector. For the cited examples, \code{x} must be given
 #'  as follows \code{c(p="1", theta="alpha", theta="beta")}. Note the repeated name \code{theta} which must be provided for every component of the list \code{theta} by which a
-#'  differerentiation is required.
+#'  differentiation is required.
 #' @param env An environment where the symbols and functions are searched for.
 #'  Defaults to \code{parent.frame()} for \code{f} expression and to
 #'  \code{environment(f)} if \code{f} is a function. For primitive function,
@@ -30,7 +30,7 @@
 #' @param use.D An optional logical (default FALSE), indicates if base::D()
 #'  must be used for differentiation of basic expressions.
 #' @param cache.exp An optional logical (default TRUE), indicates if
-#'  final expression must be optimized with cached subexpressions.
+#'  final expression must be optimized with cached sub-expressions.
 #'  If enabled, repeated calculations are made only once and their
 #'  results stored in cache variables which are then reused.
 #' @param nderiv An optional integer vector of derivative orders to calculate.
@@ -96,10 +96,11 @@
 #'      be differentiated (cf. examples section).
 #'  \item in case of multiple derivatives (e.g. gradient and hessian calculation),
 #'      caching can make calculation economies for both
+#'  \item Starting from v4.0, some matrix calculus operations are possible (contribution of Andreas Rappold). See an example hereafter for differentiation of the inverse of 2x2 matrix and whose elements depend on variable of differentiation \code{x}.
 #' }
 #'
-#' Two work environments \code{drule} and \code{simplifications} are
-#' exported in the package namescape.
+#' Two environments \code{drule} and \code{simplifications} are
+#' exported in the package's NAMESPACE.
 #' As their names indicate, they contain tables of derivative and
 #' simplification rules.
 #' To see the list of defined rules do \code{ls(drule)}.
@@ -109,6 +110,10 @@
 #' per argument. See \code{drule$log} for an example to follow.
 #' After adding \code{sinpi} you can differentiate expressions like
 #' \code{Deriv(~ sinpi(x^2), "x")}. The chain rule will automatically apply.
+#' 
+#' Starting from v4.0, user can benefit from a syntax \code{.d_X} in the rule writing. Here \code{X} must be replace by an argument name (cf. \code{drule[["solve"]]} for an example). A use of this syntax leads to a replacement of this place-holder by a derivative of the function (chain rule is automatically integrated) by the named argument.
+#' \cr
+#' Another v4.0 novelty in rule's syntax is a possible use of optional parameter \code{`_missing`} which can be set to TRUE or FALSE (default) to indicate how to treat missing arguments. By default, i.e. in absence of this parameter or set to FALSE, missing arguments were replaced by their default values. Now, if \code{`_missing`=TRUE} is specified in a rule, the missing arguments will be left missed in the derivative. Look \code{drule[["solve"]]} for an example.
 #'
 #' NB. In \code{abs()} and \code{sign()} function, singularity treatment
 #'     at point 0 is left to user's care.
@@ -183,8 +188,8 @@
 #' }
 #' # c(xx = 2 * xx, yy = 2 * yy)
 #'
-#' # Automatic differentiation (AD), note itermediate variable 'd' assignment
-#' \dontrun{Deriv(~{d <- ((x-m)/s)^2; exp(-0.5*d)}, "x")}
+#' # Automatic differentiation (AD), note intermediate variable 'd' assignment
+#' \dontrun{Deriv(~{d <- ((x-m)/s)^2; exp(-0.5*d)}, "x", cache.exp=FALSE)}
 #' #{
 #' #   d <- ((x - m)/s)^2
 #' #   .d_x <- 2 * ((x - m)/s^2)
@@ -193,13 +198,13 @@
 #'
 #' # Custom derivation rule
 #' \dontrun{
-#'   myfun <- function(x, y=TRUE) NULL # do something usefull
+#'   myfun <- function(x, y=TRUE) NULL # do something useful
 #'   dmyfun <- function(x, y=TRUE) NULL # myfun derivative by x.
 #'   drule[["myfun"]] <- alist(x=dmyfun(x, y), y=NULL) # y is just a logical
 #'   Deriv(myfun(z^2, FALSE), "z")
 #'   # 2 * (z * dmyfun(z^2, FALSE))
 #' }
-#' # Differentiantion by list components
+#' # Differentiation by list components
 #' \dontrun{
 #'   theta <- list(m=0.1, sd=2.)
 #'   x <- names(theta)
@@ -209,6 +214,11 @@
 #' #  (x - theta$m)/theta$sd, theta_sd = 2 * (exp(-((x - theta$m)^2/
 #' #  (2 * theta$sd))) * (x - theta$m)^2/(2 * theta$sd)^2))
 #' }
+#' # Differentiation in matrix calculus
+#' \dontrun{
+#'  Deriv(~solve(matrix(c(1, x, x**2, x**3), nrow=2, ncol=2)))
+#' }
+
 
 Deriv <- function(f, x=if (is.function(f)) NULL else all.vars(if (is.character(f)) parse(text=f) else f), env=if (is.function(f)) environment(f) else parent.frame(), use.D=FALSE, cache.exp=TRUE, nderiv=NULL, combine="c") {
 	tf <- try(f, silent=TRUE)
@@ -229,7 +239,7 @@ Deriv <- function(f, x=if (is.function(f)) NULL else all.vars(if (is.character(f
 	if (is.null(env))
 		env <- .GlobalEnv
 	if (is.null(x)) {
-		# primitive function or function given by a list membr or alike
+		# primitive function or function given by a list member or alike
 		af <- formals(args(f))
 		x <- names(af)
 		rule <- drule[[fch]]
@@ -376,8 +386,16 @@ Deriv_ <- function(st, x, env, use.D, dsym, scache, combine="c") {
 	is_sub_x <- is_index_expr &&
 				format1(st[[2]]) == nm_x && format1(st[[3]]) == x
 #browser()
-	if (is.conuloch(st) || (is_index_expr && !is_sub_x)) {
+	if (is.conuloch(st)) {
 		return(0)
+	} else if (is_index_expr && !is_sub_x) {
+#browser()
+		st[[2]] <- Deriv_(st[[2]], x, env, use.D, dsym, scache)
+		if (identical(st[[2]], 0) || identical(st[[2]], 0L)) {
+			return(0)
+		} else {
+			return(Simplify(st, scache=scache))
+		}
 	} else if (length(x) == 1 && (is.symbol(st) || (get_sub_x && is_index_expr))) {
 #browser()
 		stch <- format1(st)
@@ -416,7 +434,7 @@ Deriv_ <- function(st, x, env, use.D, dsym, scache, combine="c") {
 						dsym$l[[x[ix]]] <- list()
 				}
 			}
-			# collect defined var names (to avoid redifferentiation)
+			# collect defined var names (to avoid re-differentiation)
 			defs <- sapply(args, function(e) if (is.assign(e)) format1(e[[2]]) else "")
 #			alva <- list()
 			last_res <- list()
@@ -426,7 +444,7 @@ Deriv_ <- function(st, x, env, use.D, dsym, scache, combine="c") {
 				if (is.assign(a)) {
 					if (!is.symbol(a[[2]]))
 						stop(sprintf("In AD mode, don't know how to deal with a non symbol '%s' at lhs", format1(a[[2]])))
-					# put in scache the assignement
+					# put in scache the assignment
 					ach <- format1(a[[2]])
 					for (ix in seq_along(x)) {
 						d_ach <- paste(".", ach, "_", x[ix], sep="")
@@ -548,7 +566,13 @@ Deriv_ <- function(st, x, env, use.D, dsym, scache, combine="c") {
 		mc <- as.list(match.call(definition=da, call=st, expand.dots=FALSE))[-1]
 		da <- as.list(da)
 		da <- da[-length(da)] # all declared arguments with default values
-		aa <- modifyList(da, mc) # all arguments with actual values
+		if (isTRUE(rule$`_missing`)) {
+			aa <- mc
+			aa[setdiff(names(da), names(mc))] <- list(alist(x=)$x) # missing arguments are set missing
+		} else {
+			aa <- modifyList(da, mc) # all arguments with actual values
+		}
+		rule$`_missing`=NULL
 		# actualize the rule with actual arguments
 		rule <- lapply(rule, function(r) do.call("substitute", list(r, aa)))
 #browser()
@@ -561,21 +585,28 @@ Deriv_ <- function(st, x, env, use.D, dsym, scache, combine="c") {
 			return(0)
 		}
 		dargs <- lapply(names(rule), function(nm_a) if (is.null(mc[[nm_a]])) 0 else Deriv_(mc[[nm_a]], x, env, use.D, dsym, scache))
-		ize <- sapply(dargs, identical, 0)
+		names(dargs) <- names(rule)
+		ize <- sapply(dargs, identical, 0) | sapply(dargs, identical, matrix(0))
 		dargs <- dargs[!ize]
 		rule <- rule[!ize]
 		if (length(rule) == 0) {
 			return(0)
 		}
+#browser()
+		# actualize the rule with differentiated arguments
+		lrep=structure(dargs, names=paste0('.d_', names(dargs)))
+		rule <- lapply(rule, methods::substituteDirect, lrep)
 
 		# apply chain rule where needed
-		ione <- sapply(dargs, identical, 1)
-		imone <- sapply(dargs, identical, -1)
-		for (i in seq_along(rule)[!(ione|imone)]) {
-			rule[[i]] <- Simplify(call("*", dargs[[i]], rule[[i]]), scache=scache)
-		}
-		for (i in seq_along(rule)[imone]) {
-			rule[[i]] <- Simplify(call("-", rule[[i]]), scache=scache)
+		if (! stch %in% c("matrix", "%*%", "det", "solve", "diag")) {
+			ione <- sapply(dargs, identical, 1)
+			imone <- sapply(dargs, identical, -1)
+			for (i in seq_along(rule)[!(ione|imone)]) {
+				rule[[i]] <- Simplify(call("*", dargs[[i]], rule[[i]]), scache=scache)
+			}
+			for (i in seq_along(rule)[imone]) {
+				rule[[i]] <- Simplify(call("-", rule[[i]]), scache=scache)
+			}
 		}
 		return(Simplify(li2sum(rule), scache=scache))
 	} else if (is.function(st)) {
@@ -593,13 +624,14 @@ Deriv_ <- function(st, x, env, use.D, dsym, scache, combine="c") {
 	}
 }
 
+arg_missing <- function(x) missing(x)
 drule <- new.env()
 
 # linear functions, i.e. d(f(x))/dx == f(d(arg)/dx)
 dlin=c("+", "-", "c", "t", "sum", "cbind", "rbind", "list")
 
 # rule table
-# arithmetics
+# arithmetic
 drule[["*"]] <- alist(e1=e2, e2=e1)
 drule[["^"]] <- alist(e1=e2*e1^(e2-1), e2=e1^e2*log(e1))
 drule[["/"]] <- alist(e1=1/e2, e2=-e1/e2^2)
@@ -656,7 +688,18 @@ drule[["dnorm"]] <- alist(x=-(x-mean)/sd^2*if (log) 1 else dnorm(x, mean, sd),
 	sd=(((x - mean)/sd)^2 - 1)/sd * if (log) 1 else dnorm(x, mean, sd),
 	log=NULL)
 drule[["pnorm"]] <- alist(q=dnorm(q, mean, sd)*(if (lower.tail) 1 else -1)/(if (log.p) pnorm(q, mean, sd, lower.tail) else 1), mean=dnorm(q, mean, sd)*(if (lower.tail) -1 else 1)/(if (log.p) pnorm(q, mean, sd, lower.tail) else 1), sd=dnorm(q, mean, sd)*(mean-q)/sd*(if (lower.tail) 1 else -1)/(if (log.p) pnorm(q, mean, sd, lower.tail) else 1), lower.tail=NULL, log.p=NULL)
+drule[["qnorm"]] = alist(p=(if(lower.tail) 1 else -1)*(if(log.p) exp(p) else 1)/dnorm(qnorm(p, mean=mean, sd=sd, lower.tail=lower.tail, log.p=log.p), mean=mean, sd=sd),
+                         mean=1,
+                         sd=(qnorm(p, mean=mean, sd=sd, lower.tail=lower.tail, log.p=log.p) - mean)/sd)
 # data mangling
 #drule[["rep"]] <- alist(x=rep(1, ...)) # cannot handle '...' yet
 drule[["rep.int"]] <- alist(x=rep.int(1, times), times=NULL)
 drule[["rep_len"]] <- alist(x=rep_len(1, length.out), length.out=NULL)
+drule[["length"]] <- alist() # derivative is always 0
+# matrix calculus
+drule[["matrix"]] <- alist(`_missing`=TRUE, data=matrix(.d_data, nrow=nrow, ncol=ncol, byrow=byrow, dimnames=dimnames))
+drule[["%*%"]] <- alist(x=.d_x%*%y, y=x%*%.d_y)
+drule[["det"]] <- alist(x=det(x)*sum(diag(as.matrix(solve(x, .d_x)))))
+drule[["solve"]] <- alist(`_missing`=TRUE, a=-solve(a)%*%.d_a%*%solve(a, b),
+                         b=solve(a, .d_b))
+drule[["diag"]] = alist(`_missing`=TRUE, x=if (!is.matrix(x) && length(x) == 1 && arg_missing(nrow) && arg_missing(ncol)) matrix(0, nrow=x, ncol=x) else diag(x=.d_x, nrow, ncol, names=names))
